using System;
using System.Data;
using System.Drawing;
using TASKView.app;

namespace TASKView.lib
{
	/**
	 *  The ChartsPanel controls display of the active charts. 
	 * 
	 * @author      Martin Turon
	 * @version     2004/4/20    mturon      Initial version
	 * 
	 * $Id: classChartsPanel.cs,v 1.7 2004/04/28 23:37:44 mturon Exp $
	 */
	public class ChartsPanel : System.Windows.Forms.Panel
	{
		private const  int		CHART_COUNT = 3;
		private static Random	m_rand;

		#region // ==================== INSTANCE DATA ========================
		/** The actual array of chart objects */
		private	AxMSFlexGridLib.AxMSFlexGrid	m_legendGrid;
		private	AxCWUIControlsLib.AxCWGraph		m_stripChart0;
		private	AxCWUIControlsLib.AxCWGraph		m_stripChart1;
		private	AxCWUIControlsLib.AxCWGraph		m_stripChart2;
		private	AxCWUIControlsLib.AxCWGraph[]	m_stripChart;
		private string[]						m_chartSensor;

		protected int			m_chartCount;		//!< visible chart count
		protected int			m_chartTotal;		//!< total chart count

		#endregion

		// ======================= METHODS ===========================

		#region // ==> ChartPanel: Initialization Code
		/** Constructor -- call InitializeComponent to build GUI. */
		public ChartsPanel()
		{

			InitializeComponent();

			m_rand = new Random();

			//
			// Other constructor logic that isn't autogenerated.
			//

			m_chartSensor   = new string[CHART_COUNT];
			m_stripChart    = new AxCWUIControlsLib.AxCWGraph[CHART_COUNT];
			m_stripChart[0] = m_stripChart0;
			m_stripChart[1] = m_stripChart1;
			m_stripChart[2] = m_stripChart2;

			m_stripChart[0].Visible = false;
			m_stripChart[1].Visible = false;
			m_stripChart[2].Visible = false;

			m_chartSensor[0] = "(none)";
			m_chartSensor[1] = "(none)";
			m_chartSensor[2] = "(none)";

			m_chartCount = 0;
		}

		/** Creates the user interface -- grabbed from Form builder */
		void InitializeComponent() 
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ChartsPanel));
			this.m_legendGrid = new AxMSFlexGridLib.AxMSFlexGrid();
			this.m_stripChart0 = new AxCWUIControlsLib.AxCWGraph();
			this.m_stripChart1 = new AxCWUIControlsLib.AxCWGraph();
			this.m_stripChart2 = new AxCWUIControlsLib.AxCWGraph();
			((System.ComponentModel.ISupportInitialize)(this.m_legendGrid)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.m_stripChart0)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.m_stripChart1)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.m_stripChart2)).BeginInit();
			this.SuspendLayout();
			// 
			// m_legendGrid
			// 
			this.m_legendGrid.Location = new System.Drawing.Point(287, 17);
			this.m_legendGrid.Name = "m_legendGrid";
			this.m_legendGrid.OcxState = ((System.Windows.Forms.AxHost.State)(resources.GetObject("m_legendGrid.OcxState")));
			this.m_legendGrid.Size = new System.Drawing.Size(48, 176);
			this.m_legendGrid.TabIndex = 24;
			// 
			// m_stripChart0
			// 
			this.m_stripChart0.Location = new System.Drawing.Point(17, 17);
			this.m_stripChart0.Name = "m_stripChart0";
			this.m_stripChart0.OcxState = ((System.Windows.Forms.AxHost.State)(resources.GetObject("m_stripChart0.OcxState")));
			this.m_stripChart0.Size = new System.Drawing.Size(160, 120);
			this.m_stripChart0.TabIndex = 21;
			// 
			// m_stripChart1
			// 
			this.m_stripChart1.Location = new System.Drawing.Point(152, 17);
			this.m_stripChart1.Name = "m_stripChart1";
			this.m_stripChart1.OcxState = ((System.Windows.Forms.AxHost.State)(resources.GetObject("m_stripChart1.OcxState")));
			this.m_stripChart1.Size = new System.Drawing.Size(160, 120);
			this.m_stripChart1.TabIndex = 22;
			// 
			// m_stripChart2
			// 
			this.m_stripChart2.Location = new System.Drawing.Point(421, 17);
			this.m_stripChart2.Name = "m_stripChart2";
			this.m_stripChart2.OcxState = ((System.Windows.Forms.AxHost.State)(resources.GetObject("m_stripChart2.OcxState")));
			this.m_stripChart2.Size = new System.Drawing.Size(160, 120);
			this.m_stripChart2.TabIndex = 23;
			// 
			// ChartsPanel
			// 
			this.Controls.Add(this.m_legendGrid);
			this.Controls.Add(this.m_stripChart0);
			this.Controls.Add(this.m_stripChart1);
			this.Controls.Add(this.m_stripChart2);
			this.Dock = System.Windows.Forms.DockStyle.Fill;
			((System.ComponentModel.ISupportInitialize)(this.m_legendGrid)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.m_stripChart0)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.m_stripChart1)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.m_stripChart2)).EndInit();
			this.ResumeLayout(false);

		}

		#endregion

		#region // ==> ChartPanel: Sizing Code
		protected void RecalcSize() 
		{
			m_legendGrid.Height = this.Height;
			m_legendGrid.Top = 0;
			m_legendGrid.Left = this.Width - m_legendGrid.Width;

			m_stripChart[0].Left = 0;
			m_stripChart[0].Width = m_legendGrid.Left;
			m_stripChart[1].Left = 0;
			m_stripChart[1].Width = m_legendGrid.Left;
			m_stripChart[2].Left = 0;
			m_stripChart[2].Width = m_legendGrid.Left;

			int chartHeight = (m_chartCount > 0)  ? (int)this.Height / m_chartCount : 0;
			m_stripChart[0].Height = (m_stripChart[0].Visible) ? chartHeight : 0;
			m_stripChart[1].Height = (m_stripChart[1].Visible) ? chartHeight : 0;
			m_stripChart[2].Height = (m_stripChart[2].Visible) ? chartHeight : 0;

			m_stripChart[0].Top = 0;
			m_stripChart[1].Top = m_stripChart[0].Top + m_stripChart[0].Height;
			m_stripChart[2].Top = m_stripChart[1].Top + m_stripChart[1].Height;
		}

		protected override void OnResize(EventArgs e)
		{
			RecalcSize();
			base.OnResize (e);
		}

		#endregion

		/** 
		 *	Builds UI to display node ids and colors for all motes that are 
		 *	marked for plotting. 
		 * 
		 * @author      Martin Turon
		 * @version     2004/4/21    mturon      Initial version
		 */
		public void BuildLegend() 
		{
			int plotCount = 0;

			m_legendGrid.Cols = 1;
			m_legendGrid.Row  = 0;
			m_legendGrid.Col  = 0;
			m_legendGrid.set_ColAlignment(0, 1);
			m_legendGrid.Text = "Node";

			MoteTable moteTable = theMoteTable.Instance;
			m_legendGrid.Rows = moteTable.Count + 1;
			foreach (MoteInfo moteInfo in moteTable.Values) 
			{
				if ((moteInfo.m_flags & MoteFlags.MF_PLOT) > 0)
				{
					m_legendGrid.Row  = ++plotCount;
					m_legendGrid.Text = moteInfo.m_nodeid.ToString();
					m_legendGrid.CellBackColor = moteInfo.m_color;
				}
			}
			m_legendGrid.Rows = plotCount+1;
		}

		/** 
		 *	Fills a plot data array with random time and data information. 
		 * 
		 * @author      Martin Turon
		 * @version     2004/4/22    mturon      Initial version
		 */
		public void FillRandom(out double[] xData, out double[] yData) 
		{
			uint i;
			const int SIZE = 100;
			xData = new double[SIZE];
			yData = new double[SIZE];

			for (i=0; i<SIZE; i++) 
			{
				xData[i] = (i + 0.1) * 0.1;					// time 
				yData[i] = i + 1.0 + m_rand.Next()%100.0;	// data
			}
		}

		/** 
		 *	Fills a plot data array with time information and data readings
		 *  from a given node and sensor. 
		 * 
		 * @author      Martin Turon
		 * @version     2004/4/27    mturon      Initial version
		 */
		public int FillNode(MoteInfo moteInfo, string sensor, out double[] xData, out double[] yData) 
		{
			ushort nodeid = moteInfo.m_nodeid;
			
			OdbcManager db = theOdbcManager.Instance;
			db.Connect();
			DataSet dSet = db.CreateDataSet(
			db.GetNodeDataCommand(nodeid, sensor));	
			db.Disconnect();

			int size = 0, i = 0;
			try 
			{
				size = dSet.Tables[0].Rows.Count - 1;
				if (size < 0) size = 0;

				xData = new double[size];
				yData = new double[size];
			
				foreach (DataRow dRow in dSet.Tables[0].Rows)
				{
					try 
					{
						if (i >= size) break;
						DateTime rtime = Convert.ToDateTime(dRow["result_time"]);
						object data = dRow[sensor];
						if (data.GetType() == Type.GetType("System.DBNull")) continue;
						yData[i] = Convert.ToDouble(data);
						xData[i] = rtime.ToOADate();
						++i;
					} 
					catch (Exception ex) 
					{
						theOdbcManager.Instance.ErrorLog("\n" + ex.ToString());
					}
				}	
			} 
			catch (Exception ex) 
			{
				theOdbcManager.Instance.ErrorLog("\n" + ex.ToString());
				xData = new double[0];
				yData = new double[0];
			}
	
			return size;
		}

		/** 
		 *	Fills the given chart with data from the given sensor. 
		 * 
		 * @author      Martin Turon
		 * @version     2004/4/28    mturon      Initial version
		 */
		public void ChartUpdate(uint chartNum) 
		{
			AxCWUIControlsLib.AxCWGraph	chartThis = m_stripChart[chartNum];
			string sensor = m_chartSensor[chartNum];

			if (!chartThis.Visible) return;

			uint i = 1;
			double[] xData, yData;

			foreach (MoteInfo moteInfo in theMoteTable.Instance.Values) 
			{
				if ((moteInfo.m_flags & MoteFlags.MF_PLOT) > 0)
				{
					int nodeid = moteInfo.m_nodeid;

					if (moteInfo.m_plot[chartNum] > 0) 
					{
						moteInfo.m_plot[chartNum] = i++;	// store new index
						continue;
					}

					//	FillRandom(out xData, out yData);
					if (FillNode(moteInfo, sensor, out xData, out yData) == 0) continue;

					// The colors red and blue are flipped in the VB6 activeX chart component...
					// This code provides a good example of how to convert Color objects
					// into unsigned integer and back in C#.  See also MoteTable.m_colors.
					Color color			= moteInfo.m_color;
					Color colorConvert	= Color.FromArgb(color.B, color.G, color.R);
					uint  colorPlot		= (uint)colorConvert.ToArgb();

					chartThis.Plots.Add();
					chartThis.Plots.Item(i).LineColor = colorPlot;
					chartThis.Plots.Item(i).ChartXvsY(xData, yData);

					moteInfo.m_plot[chartNum] = i++;
				} 				
				else 
				{
					uint plotNum = moteInfo.m_plot[chartNum];
					if (plotNum > 0) 
					{
						chartThis.Plots.Remove(plotNum);
						moteInfo.m_plot[chartNum] = 0;
					}
				}
			}
		}

		/** 
		 *	Toggles the visibility of the given chart, and refills it
		 *	with data from the given sensor. 
		 * 
		 * @author      Martin Turon
		 * @version     2004/4/21    mturon      Initial version
		 */
		public void ChartSensor(uint chartNum, string sensor) 
		{
			AxCWUIControlsLib.AxCWGraph	chartThis = m_stripChart[chartNum];
			bool						chartRedraw = false;

			if (theSensorTable.Instance.ContainsKey(sensor)) 
			{			
				if (!chartThis.Visible) 
				{
					m_chartCount++;
					chartThis.Visible = true;
					chartRedraw = true;
				}

				if (m_chartSensor[chartNum] != sensor) 
					chartRedraw = true;
				m_chartSensor[chartNum] = sensor;

				if (chartRedraw) 
					chartThis.Plots.RemoveAll();
			} 
			else 
			{
				if (chartThis.Visible) 
				{
					m_chartCount--;
					chartThis.Visible = false;
					chartRedraw = true;
				}
			}

			ChartUpdate(chartNum);

			if (chartRedraw) 
			{
				chartThis.Axes.Item(1).AutoScaleNow();
				chartThis.Axes.Item(2).AutoScaleNow();
				RecalcSize();
				Refresh();
			}
		}

		public void ToolZoom() 
		{
			for (int i=0; i < CHART_COUNT; i++) 
			{
				m_stripChart[i].TrackMode = 
					CWUIControlsLib.CWGraphTrackModes.cwGTrackZoomRectXY;
			}

			// set new mouse icon
			//this.Mouse FrmDataGraph.MousePointer = vbCrosshair
		}

		public void ToolPan() 
		{
			for (int i=0; i < CHART_COUNT; i++) 
			{
				m_stripChart[i].TrackMode = 
					CWUIControlsLib.CWGraphTrackModes.cwGTrackPanPlotAreaXY;
			}
			//		' set new mouse icon
			//		FrmDataGraph.MousePointer = vbSizeAll
		}

		public void ToolZoomOut() 
		{
			for (int i=0; i < CHART_COUNT; i++) 
			{
				m_stripChart[i].Axes.Item(1).AutoScaleNow();
				m_stripChart[i].Axes.Item(2).AutoScaleNow();
				m_stripChart[i].TrackMode = 
					CWUIControlsLib.CWGraphTrackModes.cwGTrackAllEvents;
			}
		}

	}	// class ChartPanel


	/**
	 *  The ChartComboBox class controls which sensor to chart. 
	 * 
	 * @author      Martin Turon
	 * @version     2004/4/21    mturon      Initial version
	 */
	public class ChartComboBox : System.Windows.Forms.ComboBox
	{
		public ChartComboBox() 
		{
			try 
			{
				this.Items.Clear();
				this.Items.Add("(none)");
				Initialize();
			} 
			catch (Exception ex) 
			{
				theOdbcManager.Instance.ErrorLog(ex.ToString());
			}
		}

		private void Initialize() 
		{
			foreach (SensorInfo sensor in theSensorTable.Instance.Values) 
			{
				this.Items.Add(sensor.m_name);
			}
		}
	}	// class ChartComboBox
	
}
