// $Id: MatlabControl.java,v 1.1.14.1 2003/08/18 22:09:43 cssharp Exp $

/* MatlabControl.java
 *
 * "Copyright (c) 2001 and The Regents of the University 
 * of California.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * $\Id$
 */

/**
 * This class interfaces with the current Matlab session, allowing you 
 * to call matlab commands from Java objects
 *
 * @author <a href="mailto:kamin@cs.berkeley.edu">Kamin Whitehouse</a>
 */

package net.tinyos.matlab;

import java.lang.System.*;
import com.mathworks.jmi.*;

public class MatlabControl
{
    static Matlab matlab = null; //this is the com.mathworks.jmi.Matlab class,which has functionality allowing one to interact with the matlab session.

    public MatlabControl ( )
    {
	try{
	    if(matlab==null)
		matlab=new Matlab();//this command links to the current matlab session
	}
	catch(Exception e){}
    }

    /**Evaluate a string, Matlab script, or Matlab function**/
    public void eval(String Command){
	Matlab.whenMatlabReady(new MatlabEvalCommand(Command));
    }

    /**Evaluate a Matlab function that requires arguments.  Each element of
       the "args" vector is an argument to the function "Command"**/
    public void feval(String Command, Object[] args)
    {
	Matlab.whenMatlabReady(new MatlabFevalCommand(Command, args));
    }

    /** This class is used to execute a string in Matlab **/
    protected class MatlabEvalCommand implements Runnable
    {
	String command;
	
	public MatlabEvalCommand(String Command) {
	    command=Command;
	}
	
	public void run() {
		try {
		    matlab.evalConsoleOutput(command);
		} catch (Exception e) {
		    System.out.println(e.toString());
		}
	}
    }
    
    /** This class is used to execute a function in matlab and pass paramseters**/
    protected class MatlabFevalCommand implements Runnable
    {
	String command;
	Object[] args;
	
	public MatlabFevalCommand(String Command, Object[] Args) {
	    command=Command;
	    args = Args;
	}
	
	public void run() {
		try {
		    matlab.fevalConsoleOutput(command, args, 0, null);
		} catch (Exception e) {
		    System.out.println(e.toString());
		}
	}
    }
    
}





