// $Id: SimObjectGenerator.java,v 1.2.8.3 2003/08/26 09:08:11 cssharp Exp $

/*									tab:2
 *
 *
 * "Copyright (c) 2000 and The Regents of the University 
 * of California.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose, without fee, and without written
 * agreement is hereby granted, provided that the above copyright
 * notice and the following two paragraphs appear in all copies of
 * this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Authors:	Nelson Lee
 * Date:        December 05, 2002
 * Desc:        
 *
 */

/**
 * @author Nelson Lee
 */


package net.tinyos.sim;

import java.awt.event.*;
import javax.swing.*;
import net.tinyos.sim.event.*;

public abstract class SimObjectGenerator implements SimConst {
    TinyViz tv;
    CoordinateTransformer cT;
    SimEventBus eventBus;
    MotePanel motePanel;
    SimState state;
    
    SimObjectPopupMenu popup;
    

    // typically, each object generator can be accessed by only 1 plugin.
    // thus, with the exception of MoteSimObjectGenerators, all object generators
    // should be given a plugin, to alert when the object has been created.
    public SimObjectGenerator(TinyViz tv) {
	this.tv = tv;
	this.cT = tv.getCoordTransformer();
	this.state = tv.getSimState();
	this.eventBus = tv.getEventBus();
	this.motePanel = tv.getMotePanel();
	this.popup = new SimObjectPopupMenu(tv, this);
    }

    public void registerMenuItem(JMenuItem menuItem) {
	synchronized (state) {
	    popup.add(menuItem);
	}
    }
    
    public void deregisterMenuItem(JMenuItem menuItem) {
	synchronized (state) {
	    popup.remove(menuItem);
	}
    }
    
    public SimObjectPopupMenu getPopupMenu() {
	synchronized (state) {
	    return popup;
	}
    }
    
    public abstract void mouseAction(MouseEvent e);

}
