function serialForward = connectToSerial(serial)
%CONNECTTOSERIAL
%
% usage: serialForward = connectToSerial('COM1')
%
%This function takes a string and connects to that serial port.
%
%If the connection already exists, it returns the serialForward object that did
%it.
%
%IF the connection cannot be made, an error is given.

global COMM

if ~ischar(serial) | ~strncmpi(serial, 'COM', 3) | length(serial)<4
    warning(' usage: connectToSerial(''COM1'')');
end

TF=strcmp(COMM.serials, serial);
if any(TF)
    serialForward=COMM.serialForward{TF};
    return
end

try
    port=str2num(serial(4:end));
    port=num2str(port+9000);
    serialForward = net.tinyos.sf.SerialForward;
    serialForward.main({'-comm',serial,'-port',port,'-baud',num2str(COMM.baud),'-no-gui','-quiet'});
    COMM.serials{end+1}=serial;
    COMM.serialForward{end+1}=serialForward;
catch
    serialForward=[];
    warning(['Could not connect: ' serial])
end

    
    
    
    